/**********************************************************************
	"Copyright 1990-2018 Brian MacWhinney. Use is subject to Gnu Public License
	as stated in the attached "gpl.txt" file."
*/


#define CHAT_MODE 1

#include "cu.h"
#include "c_curses.h"

#ifdef UNX
	#define RGBColor int
#endif

#if !defined(UNX)
#define _main flucalc_main
#define call flucalc_call
#define getflag flucalc_getflag
#define init flucalc_init
#define usage flucalc_usage
#endif

#define IS_WIN_MODE FALSE
#include "mul.h" 

extern char OverWriteFile;
extern char outputOnlyData;
extern char isLanguageExplicit;
extern char linkMain2Mor;
extern struct tier *defheadtier;
extern struct tier *headtier;

// isPOSMatch beg
struct flucalc_lines {
	char isUsed;
	char *loc;
	char *tier;
	flucalc_lines *nextLine;
} ;

struct flucalc_where {
	unsigned int count;
	struct flucalc_lines *line;
	struct flucalc_where *nextMatch;
} ;

struct flucalc_freqTnode {
	char *word;
	struct flucalc_where *where;
	unsigned int count;
	struct flucalc_freqTnode *left;
	struct flucalc_freqTnode *right;
};
// isPOSMatch end

struct flucalc_speakers {
	char isSpeakerFound;
	char isMORFound;
	char isPSDFound; // +/.

	char *fname;
	char *sp;
	char *ID;

	// isPOSMatch beg
	struct flucalc_freqTnode *words;
	long total;				/* total number of words */
	long different;			/* number of different words */
	// isPOSMatch end


	float tm;
	float morUtt;
	float morWords;
	float morSyllables;

	float prolongation;	// :
	float broken_word;	// ^
	float block;		// ≠
	float PWR;			// ↫ ↫ pairs//Part Word Repetition; Sequences
	float PWRRU;		// ↫ - - ↫  // iterations
	float phono_frag;	// &+
	float WWR;			// WWR:             how many times one word "the [/] the [/] the" occurs. In this case it is 1.
	float mWWR;			// mono-WWR:        how many times monosyllable "the [/] the [/] the" occurs. In this case it is 1.
	float WWRRU;		// WWR-RU:      [/] Whole Word repetition iterations
	float mWWRRU;		// mono-WWR-RU: [/] Whole monosyllable Word repetition iterations
	float phrase_repets;// Phrase repetitions: <> [/]
	float word_revis;	// [//]
	float phrase_revis;	// <> [//]
	float pauses_cnt;	// (.)
	float pauses_dur;	// (2.4)
	float filled_pauses;// &-
	float openClass;
	float closedClass;

	struct flucalc_speakers *next_sp;
} ;

struct flucalc_tnode {
	char *word;
	int count;
	struct flucalc_tnode *left;
	struct flucalc_tnode *right;
};

static char flucalc_ftime, isSyllWordsList, isWordMode, isPOSMatch, isMorTierFirst;
static struct flucalc_speakers *sp_head;
static struct flucalc_tnode *rootWords;
static flucalc_lines *rootLines;
static FILE *SyllWordsListFP;


void usage() {
	puts("FLUCALC creates a spreedsheet with a series of fluency measures.");
#ifdef UNX
	printf("FluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.\n");
	printf("TO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.\n");
#else
	printf("%c%cFluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
	printf("%c%cTO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
#endif
	printf("Usage: flucalc [%s] filename(s)\n",mainflgs());
	puts("+b : select word mode analyses (default: syllables mode)");
	puts("+c5: when combined with +p option it will reverse tier's priority");
	puts("+e1: create file with words and their syllable count");
	puts("+pS: search for word S and match it to corresponding POS");
	puts("+p@F: search for words in file F and match it to corresponding POS");
	mainusage(FALSE);
#ifdef UNX
	printf("FluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.\n");
	printf("TO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.\n");
#else
	printf("%c%cFluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
	printf("%c%cTO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
#endif
	cutt_exit(0);
}

// isPOSMatch beg
static flucalc_lines *flucal_freeLines(flucalc_lines *lines, char isJustOne) {
	flucalc_lines *t;

	while (lines != NULL) {
		if (lines->loc != NULL)
			free(lines->loc);
		if (lines->tier != NULL)
			free(lines->tier);
		t = lines;
		lines = lines->nextLine;
		free(t);
		if (isJustOne)
			break;
	}
	return(lines);
}

static flucalc_where *flucal_freeWhere(flucalc_where *where) {
	flucalc_where *t;

	while (where != NULL) {
		t = where;
		where = where->nextMatch;
		free(t);
	}
	return(NULL);
}

static void flucalc_freqFreetree(struct flucalc_freqTnode *p) {
	if (p != NULL) {
		flucalc_freqFreetree(p->left);
		flucalc_freqFreetree(p->right);
		free(p->word);
		p->where = flucal_freeWhere(p->where);
		free(p);
	}
}
// isPOSMatch end

static struct flucalc_speakers *freespeakers(struct flucalc_speakers *p) {
	struct flucalc_speakers *ts;

	while (p) {
		ts = p;
		p = p->next_sp;
		if (ts->fname != NULL)
			free(ts->fname);
		if (ts->sp != NULL)
			free(ts->sp);
		if (ts->ID != NULL)
			free(ts->ID);
		flucalc_freqFreetree(ts->words);
		free(ts);
	}
	return(NULL);
}

static void flucalc_freetree(struct flucalc_tnode *p) {
	if (p != NULL) {
		flucalc_freetree(p->left);
		flucalc_freetree(p->right);
		free(p->word);
		free(p);
	}
}

static void flucalc_error(char IsOutOfMem) {
	if (IsOutOfMem)
		fputs("ERROR: Out of memory.\n",stderr);
	sp_head = freespeakers(sp_head);
	if (SyllWordsListFP != NULL)
		fclose(SyllWordsListFP);
	flucalc_freetree(rootWords);
	rootWords = NULL;
	rootLines = flucal_freeLines(rootLines, FALSE);
	cutt_exit(0);
}

// isPOSMatch beg
static flucalc_lines *flucalc_AddNewLine(flucalc_lines *lines, char *loc, char *tier) {
	flucalc_lines *p;

	if ((p=NEW(struct flucalc_lines)) == NULL)
		flucalc_error(TRUE);
	if (loc != NULL) {
		if ((p->loc=(char *)malloc(strlen(loc)+1)) == NULL)
			flucalc_error(TRUE);
		strcpy(p->loc, loc);
	} else
		p->loc = NULL;
	if ((p->tier=(char *)malloc(strlen(tier)+1)) == NULL)
		flucalc_error(TRUE);
	strcpy(p->tier, tier);
	p->nextLine = lines;
	p->isUsed = FALSE;
	lines = p;

	return(lines);
}

static char *flucalc_strsave(char *s) {
	char *p;

	if ((p=(char *)malloc(strlen(s)+1)) != NULL)
		strcpy(p, s);
	else
		flucalc_error(TRUE);
	return(p);
}

static struct flucalc_freqTnode *flucalc_talloc(char *word, unsigned int count) {
	struct flucalc_freqTnode *p;

	if ((p=NEW(struct flucalc_freqTnode)) == NULL)
		flucalc_error(TRUE);
	p->word = word;
	p->count = count;
	return(p);
}

static flucalc_where *flucalc_AddNewMatch(flucalc_where *where, flucalc_lines *line) {
	flucalc_where *p;

	if (where == NULL) {
		if ((p=NEW(struct flucalc_where)) == NULL)
			flucalc_error(TRUE);
		where = p;
	} else {
		for (p=where; p->nextMatch != NULL; p=p->nextMatch) {
			if (line->loc == NULL) {
				if (!strcmp(p->line->tier, line->tier)) {
					p->count++;
					return(where);
				}
			}
		}
		if (line->loc == NULL) {
			if (!strcmp(p->line->tier, line->tier)) {
				p->count++;
				return(where);
			}
		}
		if ((p->nextMatch=NEW(struct flucalc_where)) == NULL)
			flucalc_error(TRUE);
		p = p->nextMatch;
	}
	p->nextMatch = NULL;
	p->count = 1;
	p->line = line;
	line->isUsed = TRUE;
	return(where);
}

static void flucalc_addlineno(struct flucalc_freqTnode *p, flucalc_lines *line) {
	p->count++;
	if (line != NULL) {
		p->where = flucalc_AddNewMatch(p->where, line);
	}
}

static struct flucalc_freqTnode *flucalc_freqTree(struct flucalc_freqTnode *p, flucalc_speakers *ts, char *w, flucalc_lines *line) {
	int cond;
	struct flucalc_freqTnode *t = p;

	if (p == NULL) {
		ts->different++;
		p = flucalc_talloc(flucalc_strsave(w), 0);
		p->where = NULL;
		flucalc_addlineno(p, line);
		p->left = p->right = NULL;
	} else if ((cond=strcmp(w,p->word)) == 0)
		flucalc_addlineno(p, line);
	else if (cond < 0)
		p->left = flucalc_freqTree(p->left, ts, w, line);
	else {
		for (; (cond=strcmp(w,p->word)) > 0 && p->right != NULL; p=p->right) ;
		if (cond == 0)
			flucalc_addlineno(p, line);
		else if (cond < 0)
			p->left = flucalc_freqTree(p->left, ts, w, line);
		else
			p->right = flucalc_freqTree(p->right, ts, w, line); // if cond > 0
		return(t);
	}
	return(p);
}
// isPOSMatch end

static void flucalc_initTSVars(struct flucalc_speakers *ts) {
	ts->isMORFound = FALSE;
	ts->isPSDFound = FALSE;

	// isPOSMatch beg
	ts->words = NULL;
	ts->total = 0L;		/* total number of words */
	ts->different = 0L;	/* number of different words */
	// isPOSMatch end

	ts->tm		  = 0.0;
	ts->morUtt	  = 0.0;
	ts->morWords  = 0.0;
	ts->morSyllables= 0.0;

	ts->prolongation	= 0.0;
	ts->broken_word		= 0.0;
	ts->block			= 0.0;
	ts->PWR				= 0.0;
	ts->PWRRU			= 0.0;
	ts->phono_frag		= 0.0;
	ts->WWR				= 0.0;
	ts->mWWR			= 0.0;
	ts->WWRRU			= 0.0;
	ts->mWWRRU			= 0.0;
	ts->phrase_repets	= 0.0;
	ts->word_revis	    = 0.0;
	ts->phrase_revis    = 0.0;
	ts->pauses_cnt		= 0.0;
	ts->pauses_dur		= 0.0;
	ts->filled_pauses   = 0.0;
	ts->openClass		= 0.0;
	ts->closedClass		= 0.0;
}

static struct flucalc_speakers *flucalc_FindSpeaker(char *fname, char *sp, char *ID, char isSpeakerFound) {
	struct flucalc_speakers *ts, *tsp;

	uS.remblanks(sp);
	for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
		if (uS.mStricmp(ts->fname, fname) == 0) {
			if (uS.partcmp(ts->sp, sp, FALSE, FALSE)) {
				ts->isSpeakerFound = isSpeakerFound;
				return(ts);
			}
		}
	}
	if ((ts=NEW(struct flucalc_speakers)) == NULL)
		flucalc_error(TRUE);
	if ((ts->fname=(char *)malloc(strlen(fname)+1)) == NULL) {
		free(ts);
		flucalc_error(TRUE);
	}
	if (sp_head == NULL) {
		sp_head = ts;
	} else {
		for (tsp=sp_head; tsp->next_sp != NULL; tsp=tsp->next_sp) ;
		tsp->next_sp = ts;
	}
	ts->next_sp = NULL;
	strcpy(ts->fname, fname);
	if ((ts->sp=(char *)malloc(strlen(sp)+1)) == NULL) {
		flucalc_error(TRUE);
	}
	strcpy(ts->sp, sp);
	if (ID == NULL)
		ts->ID = NULL;
	else {
		if ((ts->ID=(char *)malloc(strlen(ID)+1)) == NULL)
			flucalc_error(TRUE);
		strcpy(ts->ID, ID);
	}
	ts->isSpeakerFound = isSpeakerFound;
	flucalc_initTSVars(ts);
	return(ts);
}

void init(char f) {
	int i;
	FNType debugfile[FNSize];
	struct tier *nt;
	IEWORDS *twd;

	if (f) {
		sp_head = NULL;
		rootLines = NULL;
		flucalc_ftime = TRUE;
		if (defheadtier != NULL) {
			if (defheadtier->nexttier != NULL)
				free(defheadtier->nexttier);
			free(defheadtier);
			defheadtier = NULL;
		}
		mor_initwords();
		isSyllWordsList = FALSE;
		SyllWordsListFP = NULL;
		rootWords = NULL;
		isWordMode = FALSE;
		isPOSMatch = FALSE;
		isMorTierFirst = TRUE;
	} else {
		if (flucalc_ftime) {
			flucalc_ftime = FALSE;
			if (chatmode) {
				maketierchoice("@ID:",'+',FALSE);
				maketierchoice("%mor:",'+',FALSE);
			} else {
				fprintf(stderr, "FluCalc can only run on CHAT data files\n\n");
				cutt_exit(0);
			}
			i = 0;
			for (nt=headtier; nt != NULL; nt=nt->nexttier) {
				if (nt->tcode[0] == '*') {
					i++;
				}
			}
			if (i != 1) {
				fprintf(stderr, "\nPlease specify only one speaker tier code with \"+t\" option.\n");
				cutt_exit(0);
			}
			for (i=0; GlobalPunctuation[i]; ) {
				if (GlobalPunctuation[i] == '!' ||
					GlobalPunctuation[i] == '?' ||
					GlobalPunctuation[i] == '.') 
					strcpy(GlobalPunctuation+i,GlobalPunctuation+i+1);
				else
					i++;
			}
			if (!isPOSMatch) {
				if (!isMorTierFirst) {
					fprintf(stderr,"The +c5 option can only be used with +p option\n");
					cutt_exit(0);
				}
				if (!f_override)
					stout = FALSE;
				AddCEXExtension = ".xls";
				combinput = TRUE;
				outputOnlyData = TRUE;
				OverWriteFile = TRUE;
			} else {
				for (twd=wdptr; twd != NULL; twd=twd->nextword) {
					if (twd->word[0] == '&') {
						break;
					}
				}
				addword('\0','\0',"+0*");
				if (twd == NULL) {
					addword('\0','\0',"+&*");
					addword('\0','\0',"+(*.*)");
				}
				addword('\0','\0',"++*");
				addword('\0','\0',"+-*");
				addword('\0','\0',"+#*");
				FilterTier = 1;
				isCreateFakeMor = TRUE;
			}
			linkMain2Mor = TRUE;
#if !defined(CLAN_SRV)
			if (isSyllWordsList) {
				strcpy(debugfile, "word_syllables.cex");
				SyllWordsListFP = fopen(debugfile, "w");
				if (SyllWordsListFP == NULL) {
					fprintf(stderr, "Can't create file \"%s\", perhaps it is opened by another application\n", debugfile);
				}
#ifdef _MAC_CODE
				else
					settyp(debugfile, 'TEXT', the_file_creator.out, FALSE);
#endif
			}
#endif // !defined(CLAN_SRV)
		}
	}
}

void getflag(char *f, char *f1, int *i) {
	char wd[1024+2];

	f++;
	switch(*f++) {
		case 'b':
			no_arg_option(f);
			OverWriteFile = FALSE;
			isWordMode = TRUE;
			break;
		case 'c':
			if (*f == '5')
				isMorTierFirst = FALSE;
			else {
				fprintf(stderr,"Invalid argument for option: %s\n", f-2);
				cutt_exit(0);
			}
			break;
		case 'e':
			if (*f == '1')
				isSyllWordsList = TRUE;
			else {
				fprintf(stderr,"Invalid argument for option: %s\n", f-2);
				cutt_exit(0);
			}
			break;
		case 'p':
//			if (*(f-2) == '+') {
				if (*f) {
					strncpy(wd, f, 1024);
					wd[1024] = EOS;
					removeExtraSpace(wd);
					uS.remFrontAndBackBlanks(wd);
					if (wd[0] == '+' || wd[0] == '~') {
						if (wd[1] == '@') {
							wd[1] = wd[0];
							wd[0] = *(f+1);
						}
					}
					if (wd[0] == '@') {
						rdexclf('s','i',wd+1);
					} else {
						if (wd[0] == '\\' && wd[1] == '@')
							strcpy(wd, wd+1);
						if ((wd[0] == '+' || wd[0] == '~') && wd[1] == '\\' && wd[2] == '@')
							strcpy(wd+1, wd+2);
						addword('s','i',wd);
					}
					isPOSMatch = TRUE;
				} else {
					fprintf(stderr,"Missing argument to option: %s\n", f-2);
					cutt_exit(0);
				}
//			}
			break;
		case 's':
			if (*f == '[' && *(f+1) == '-') {
				if (*(f-2) == '+')
					isLanguageExplicit = 2;
				maingetflag(f-2,f1,i);
			} else if ((*f == '[' && *(f+1) == '+') || ((*f == '+' || *f == '~') && *(f+1) == '[' && *(f+2) == '+')) {
				maingetflag(f-2,f1,i);
			} else {
				fprintf(stderr, "Please specify only postcodes, \"[+ ...]\", or precodes \"[- ...]\" with +/-s option.");
				cutt_exit(0);
			}
			break;
		case 't':
			if (*(f-2) == '+' && *f == '*') {
				maingetflag(f-2,f1,i);
			} else {
				fprintf(stderr, "\nPlease specify only one speaker tier code with \"+t\" option.\n");
				cutt_exit(0);
			}
			break;
		default:
			maingetflag(f-2,f1,i);
			break;
	}
}

// isPOSMatch beg
static void flucalc_printlines(struct flucalc_freqTnode *p) {
	flucalc_where *w;

	fprintf(fpout,"\n");
	for (w=p->where; w != NULL; w=w->nextMatch) {
		if (w->line->loc != NULL) {
			if (w->line->loc[0] != EOS)
				fprintf(fpout,"        %s\n", w->line->loc);
			fprintf(fpout,"      %s\n", w->line->tier);
		} else
			fprintf(fpout,"    %3u %s\n", w->count, w->line->tier);
	}
}

static void flucalc_freqTreeprint(struct flucalc_freqTnode *p) {
	if (p != NULL) {
		flucalc_freqTreeprint(p->left);
		do {
			fprintf(fpout,"%3u ",p->count);
			fprintf(fpout,"%-10s", p->word);
			flucalc_printlines(p);
			if (p->right == NULL)
				break;
			if (p->right->left != NULL) {
				flucalc_freqTreeprint(p->right);
				break;
			}
			p = p->right;
		} while (1);
	}
}
// isPOSMatch end

static void flucalc_pr_result(void) {
	char  *sFName;
	float SLD, TD, X, devNum;
	struct flucalc_speakers *ts;

	if (sp_head == NULL) {
		fprintf(stderr, "\nERROR: No speaker matching +t option found\n\n");
	}
	if (isPOSMatch) {
		for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
			if (!ts->isSpeakerFound) {
				fprintf(stderr, "\nWARNING: No data found for speaker \"%s\" in file \"%s\"\n\n", ts->sp, ts->fname);
				continue;
			}
			fprintf(fpout, "Speaker: %s\n", ts->sp);
			flucalc_freqTreeprint(ts->words);
			fprintf(fpout,"------------------------------\n");
			fprintf(fpout,"%5ld  Total number of different item types used\n", ts->different);
			fprintf(fpout,"%5ld  Total number of items (tokens)\n", ts->total);
			fprintf(fpout, "\n");
		}
		sp_head = freespeakers(sp_head);
	} else {
		excelHeader(fpout, newfname, 95);
		excelRow(fpout, ExcelRowStart);
		excelStrCell(fpout, "File");
		excelCommasStrCell(fpout, "Language,Corpus,Code,Age(Month),Sex,Group,SES,Role,Education,Custom_field");

		excelStrCell(fpout, "mor Utts");
		excelStrCell(fpout, "mor Words");
		excelStrCell(fpout, "mor syllables");
		excelStrCell(fpout, "words/min");
		excelStrCell(fpout, "syllables/min");

		excelStrCell(fpout, "# Prolongation");
		excelStrCell(fpout, "% Prolongation");
		excelStrCell(fpout, "# Broken word");
		excelStrCell(fpout, "% Broken word");
		excelStrCell(fpout, "# Block");
		excelStrCell(fpout, "% Block");
		excelStrCell(fpout, "# PWR");
		excelStrCell(fpout, "% PWR");
		excelStrCell(fpout, "# PWR-RU");
		excelStrCell(fpout, "% PWR-RU");
		excelStrCell(fpout, "# WWR");
		excelStrCell(fpout, "% WWR");
		excelStrCell(fpout, "# mono-WWR");
		excelStrCell(fpout, "% mono-WWR");
		excelStrCell(fpout, "# WWR-RU");
		excelStrCell(fpout, "% WWR-RU");
		excelStrCell(fpout, "# mono-WWR-RU");
		excelStrCell(fpout, "% mono-WWR-RU");
		excelStrCell(fpout, "Mean RU");
		excelStrCell(fpout, "# Phonological fragment");
		excelStrCell(fpout, "% Phonological fragment");
		excelStrCell(fpout, "# Phrase repetitions");
		excelStrCell(fpout, "% Phrase repetitions");
		excelStrCell(fpout, "# Word revisions");
		excelStrCell(fpout, "% Word revisions");
		excelStrCell(fpout, "# Phrase revisions");
		excelStrCell(fpout, "% Phrase revisions");
		excelStrCell(fpout, "# Pauses");
		excelStrCell(fpout, "% Pauses");
//		excelStrCell(fpout, "# Pause duration");
//		excelStrCell(fpout, "% Pause duration");
		excelStrCell(fpout, "# Filled pauses");
		excelStrCell(fpout, "% Filled pauses");

		excelStrCell(fpout, "# TD");
		excelStrCell(fpout, "% TD");
		excelStrCell(fpout, "# SLD");
		excelStrCell(fpout, "% SLD");

		excelStrCell(fpout, "# Total (SLD+TD)");
		excelStrCell(fpout, "% Total (SLD+TD)");

		excelStrCell(fpout, "SLD Ratio");

		excelStrCell(fpout, "Content/Function");

		excelStrCell(fpout, "Weighted SLD");
		excelRow(fpout, ExcelRowEnd);

		for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
			if (!ts->isSpeakerFound) {
				fprintf(stderr, "\nWARNING: No data found for speaker \"%s\" in file \"%s\"\n\n", ts->sp, ts->fname);
				continue;
			}
// ".xls"
			sFName = strrchr(ts->fname, PATHDELIMCHR);
			if (sFName != NULL)
				sFName = sFName + 1;
			else
				sFName = ts->fname;
			excelRow(fpout, ExcelRowStart);
			excelStrCell(fpout, sFName);
			if (ts->ID) {
				excelOutputID(fpout, ts->ID);
			} else {
				excelCommasStrCell(fpout, ".,.");
				excelStrCell(fpout, ts->sp);
				excelCommasStrCell(fpout, ".,.,.,.,.,.,.");
			}
			if (!ts->isMORFound || ts->morWords == 0.0 || ts->morSyllables == 0.0) {
				fprintf(stderr, "\n*** File \"%s\": Speaker \"%s\"\n", sFName, ts->sp);
				fprintf(stderr, "WARNING: Speaker \"%s\" has no \"%s\" tiers.\n\n", ts->sp, "%mor:");
				excelNumCell(fpout, "%.0f", ts->morUtt);
				excelNumCell(fpout, "%.0f", ts->morWords);
				excelNumCell(fpout, "%.0f", ts->morSyllables);
				excelCommasStrCell(fpout, "N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A");
			} else {
				excelNumCell(fpout, "%.0f", ts->morUtt);
				excelNumCell(fpout, "%.0f", ts->morWords);
				excelNumCell(fpout, "%.0f", ts->morSyllables);

				if (ts->tm == 0.0)
					excelStrCell(fpout, "N/A");
				else
					excelNumCell(fpout, "%.3f", ts->morWords / (ts->tm / 60.0000));

				if (ts->tm == 0.0)
					excelStrCell(fpout, "N/A");
				else
					excelNumCell(fpout, "%.3f", ts->morSyllables / (ts->tm / 60.0000));

				if (isWordMode)
					devNum = ts->morWords;
				else
					devNum = ts->morSyllables;

				excelNumCell(fpout, "%.3f", ts->prolongation);
				excelNumCell(fpout, "%.3f", (ts->prolongation/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->broken_word);
				excelNumCell(fpout, "%.3f", (ts->broken_word/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->block);
				excelNumCell(fpout, "%.3f", (ts->block/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->PWR);
				excelNumCell(fpout, "%.3f", (ts->PWR/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->PWRRU);
				excelNumCell(fpout, "%.3f", (ts->PWRRU/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->WWR);
				excelNumCell(fpout, "%.3f", (ts->WWR/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->mWWR);
				excelNumCell(fpout, "%.3f", (ts->mWWR/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->WWRRU);
				excelNumCell(fpout, "%.3f", (ts->WWRRU/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->mWWRRU);
				excelNumCell(fpout, "%.3f", (ts->mWWRRU/devNum)*100.0000);
				X = ts->PWR + ts->WWR;
				if (X <= 0.0)
					X = 0.0;
				else {
					X = (ts->PWRRU + ts->WWRRU) / X;
				}
				excelNumCell(fpout, "%.3f", X);
				excelNumCell(fpout, "%.3f", ts->phono_frag);
				excelNumCell(fpout, "%.3f", (ts->phono_frag/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->phrase_repets);
				excelNumCell(fpout, "%.3f", (ts->phrase_repets/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->word_revis);
				excelNumCell(fpout, "%.3f", (ts->word_revis/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->phrase_revis);
				excelNumCell(fpout, "%.3f", (ts->phrase_revis/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->pauses_cnt);
				excelNumCell(fpout, "%.3f", (ts->pauses_cnt/devNum)*100.0000);
//				excelNumCell(fpout, "%.3f", ts->pauses_dur);
//				excelNumCell(fpout, "%.3f", (ts->pauses_dur/devNum)*100.0000);
				excelNumCell(fpout, "%.3f", ts->filled_pauses);
				excelNumCell(fpout, "%.3f", (ts->filled_pauses/devNum)*100.0000);

				TD = ts->phrase_repets+ts->word_revis+ts->phrase_revis+ts->pauses_cnt+ts->pauses_dur+ts->filled_pauses;
				excelNumCell(fpout, "%.3f", TD);
				excelNumCell(fpout, "%.3f", (TD/devNum)*100.0000);

				SLD = ts->prolongation+ts->broken_word+ts->block+ts->PWR+ts->phono_frag+ts->mWWR;
				excelNumCell(fpout, "%.3f", SLD);
				excelNumCell(fpout, "%.3f", (SLD/devNum)*100.0000);

				excelNumCell(fpout, "%.3f", SLD+TD);
				excelNumCell(fpout, "%.3f", ((SLD/devNum)*100.0000)+((TD/devNum)*100.0000));

				if ((SLD+TD) == 0.0)
					excelStrCell(fpout, "N/A");
				else
					excelNumCell(fpout, "%.3f", SLD / (SLD+TD));

				if (ts->closedClass == 0.0)
					excelStrCell(fpout, "N/A");
				else
					excelNumCell(fpout, "%.3f", ts->openClass/ts->closedClass);

				X = (ts->PWR * 100.0000 / ts->morSyllables) + (ts->mWWR * 100.0000 / ts->morSyllables);
				if (X > 0) {
					SLD = (X * ( ( (ts->PWRRU * 100.0000 / ts->morSyllables) + (ts->mWWRRU * 100.0000 / ts->morSyllables) ) / X) ) +
						  (2 * ( (ts->prolongation * 100.0000 / ts->morSyllables) + (ts->block * 100.0000 / ts->morSyllables) ) );
				} else {
					SLD = (2 * ( (ts->prolongation * 100.0000 / ts->morSyllables) + (ts->block * 100.0000 / ts->morSyllables) ) );
				}
				excelNumCell(fpout, "%.3f", SLD);
			}
			excelRow(fpout, ExcelRowEnd);
		}
		excelRow(fpout, ExcelRowEmpty);
		excelRow(fpout, ExcelRowEmpty);
		excelRowOneStrCell(fpout, ExcelBlkCell, "Interpretation of weighted SLD score: Scores above 4.0 is highly suggestive of clinical diagnosis of stuttering in young children");
		excelRowOneStrCell(fpout, ExcelBlkCell, "Weighted score formula (from Ambrose & Yairi, 1999): ((PWR + mono-WWR) * ((PWR-RU + mono-WWR-RU)/(PWR + mono- WWR))) + (2 * (prologations + blocks))");
		if (isWordMode)
			excelRowOneStrCell(fpout, ExcelRedCell, "This spreadsheet was run in word mode");
		else
			excelRowOneStrCell(fpout, ExcelRedCell, "This spreadsheet was run in syllable mode");
		excelFooter(fpout);
		sp_head = freespeakers(sp_head);
	}
}

/*

 "Mean RUs" = (PWRRU + WWRRU) / (PWR + WWR)


 TD = phrase_repets+word_revis+phrase_revis+pauses_cnt+pauses_dur+filled_pauses;
 "#TD" = TD
 "%TD" = (TD/morWords)*100.0000


 SLD = prolongation+broken_word+block+PWR+phono_frag+WWR;
 "#SLD" = SLD
 "%SLD" = (SLD/morWords)*100.0000


 "#Total (SLD+TD)" = SLD + TD
 "%Total (SLD+TD)" = ((SLD/morWords)*100.0000) + ((TD/morWords)*100.0000)


 "SLD Ratio" = SLD / (SLD + TD)


 X = (PWR * 100.0000 / morSyllables) + (mWWR * 100.0000 / morSyllables)
 SLD = (X * ( ( (PWRRU * 100.0000 / morSyllables) + (mWWRRU * 100.0000 / morSyllables) ) / X) ) +
       (2 * ( (prolongation * 100.0000 / morSyllables) + (block * 100.0000 / morSyllables) ) )


 X = ( PWR + mWWR )
 If X is not equal to zero, then
	SLD = ( X * ( ( PWRRU + mWWRRU) / X  ) ) + ( 2 * ( prolongation + block ) )
 If X is equal to zero, then
	SLD = ( 2 * ( prolongation + block ) )


 SLD = ( (PWR + mWWR) * ( (PWRRU + mWWRRU) / (PWR + mWWR) ) ) + (2 * ( prolongation + block ) )
 "Weighted SLD" = SLD);
*/

static int countSyllables(char *word) {
	int i, j, startIndex, vCnt;
	char tWord[BUFSIZ+1], isRepSeg, isBullet;

	if (word[0] == '[' || word[0] == '(' || word[0] == EOS)
		return(0);
	isRepSeg = FALSE;
	isBullet = FALSE;
	j = 0;
	for (i=0; word[i] != EOS; i++) {
		if (UTF8_IS_LEAD((unsigned char)word[i]) && word[i] == (char)0xE2) {
			if (word[i+1] == (char)0x86 && word[i+2] == (char)0xAB) {
				isRepSeg = !isRepSeg;
			}
		} else if (word[i] == HIDEN_C)
			isBullet = !isBullet;
		if (!isRepSeg && !isBullet && isalnum(word[i]))
			tWord[j++] = word[i];
	}
	tWord[j] = EOS;
	if (tWord[0] == EOS)
		return(0);
	//	if (uS.mStricmp(tWord, "something") == 0)
	//		return(2);
	vCnt = 0;
	if (uS.mStrnicmp(tWord,"ice", 3) == 0) {
		vCnt = 1;
		startIndex = 3;
	} else if (uS.mStrnicmp(tWord,"some", 4) == 0 || uS.mStrnicmp(tWord,"fire", 4) == 0 ||
			   uS.mStrnicmp(tWord,"bake", 4) == 0 || uS.mStrnicmp(tWord,"base", 4) == 0 ||
			   uS.mStrnicmp(tWord,"bone", 4) == 0 || uS.mStrnicmp(tWord,"cake", 4) == 0 ||
			   uS.mStrnicmp(tWord,"care", 4) == 0 || uS.mStrnicmp(tWord,"dare", 4) == 0 ||
			   uS.mStrnicmp(tWord,"fire", 4) == 0 || uS.mStrnicmp(tWord,"fuse", 4) == 0 ||
			   uS.mStrnicmp(tWord,"game", 4) == 0 || uS.mStrnicmp(tWord,"home", 4) == 0 ||
			   uS.mStrnicmp(tWord,"juke", 4) == 0 || uS.mStrnicmp(tWord,"lake", 4) == 0 ||
			   uS.mStrnicmp(tWord,"life", 4) == 0 || uS.mStrnicmp(tWord,"mine", 4) == 0 ||
			   uS.mStrnicmp(tWord,"mole", 4) == 0 || uS.mStrnicmp(tWord,"name", 4) == 0 ||
			   uS.mStrnicmp(tWord,"nose", 4) == 0 || uS.mStrnicmp(tWord,"note", 4) == 0 ||
			   uS.mStrnicmp(tWord,"race", 4) == 0 || uS.mStrnicmp(tWord,"rice", 4) == 0 ||
			   uS.mStrnicmp(tWord,"side", 4) == 0 || uS.mStrnicmp(tWord,"take", 4) == 0 ||
			   uS.mStrnicmp(tWord,"tape", 4) == 0 || uS.mStrnicmp(tWord,"time", 4) == 0 ||
			   uS.mStrnicmp(tWord,"wine", 4) == 0 || uS.mStrnicmp(tWord,"wipe", 4) == 0) {
		vCnt = 1;
		startIndex = 4;
	} else if (uS.mStrnicmp(tWord,"goose", 5) == 0 || uS.mStrnicmp(tWord,"grade", 5) == 0 ||
			   uS.mStrnicmp(tWord,"grape", 5) == 0 || uS.mStrnicmp(tWord,"horse", 5) == 0 ||
			   uS.mStrnicmp(tWord,"house", 5) == 0 || uS.mStrnicmp(tWord,"phone", 5) == 0 ||
			   uS.mStrnicmp(tWord,"snake", 5) == 0 || uS.mStrnicmp(tWord,"space", 5) == 0 ||
			   uS.mStrnicmp(tWord,"store", 5) == 0 || uS.mStrnicmp(tWord,"stove", 5) == 0 ||
			   uS.mStrnicmp(tWord,"voice", 5) == 0 || uS.mStrnicmp(tWord,"waste", 5) == 0) {
		vCnt = 1;
		startIndex = 5;
	} else if (uS.mStrnicmp(tWord,"cheese", 6) == 0) {
		vCnt = 1;
		startIndex = 6;
	} else if (uS.mStrnicmp(tWord,"police", 6) == 0) {
		vCnt = 2;
		startIndex = 6;
	} else {
		vCnt = 0;
		startIndex = 0;
	}
	for (i=startIndex; tWord[i] != EOS; i++) {
		if ((i == startIndex || !uS.isVowel(tWord+i-1)) && uS.isVowel(tWord+i)) {
			if ((tWord[i] == 'e' || tWord[i] == 'E') && tWord[i+1] == EOS) {
				if (vCnt == 0)
					vCnt++;
				else if (i > 0 && (tWord[i-1] == 'l' || tWord[i-1] == 'L'))
					vCnt++;
			} else
				vCnt++;
		}
	}
	return(vCnt);
}

static struct flucalc_tnode *flucalc_tree(struct flucalc_tnode *p, char *word, int count) {
	int cond;
	struct flucalc_tnode *t = p;

	if (p == NULL) {
		if ((p=NEW(struct flucalc_tnode)) == NULL)
			flucalc_error(TRUE);
		p->word = (char *)malloc(strlen(word)+1);
		if (p->word == NULL)
			flucalc_error(TRUE);
		strcpy(p->word, word);
		p->count = count;
		p->left = p->right = NULL;
	} else if ((cond=strcmp(word,p->word)) < 0)
		p->left = flucalc_tree(p->left, word, count);
	else if (cond > 0){
		for (; (cond=strcmp(word,p->word)) > 0 && p->right != NULL; p=p->right) ;
		if (cond < 0)
			p->left = flucalc_tree(p->left, word, count);
		else if (cond > 0)
			p->right = flucalc_tree(p->right, word, count); /* if cond > 0 */
		return(t);
	}
	return(p);
}

static char isUttDel(char *line, int pos) {
	if (line[pos] == '?' && line[pos+1] == '|')
		;
	else if (uS.IsUtteranceDel(line, pos)) {
		if (!uS.atUFound(line, pos, &dFnt, MBF))
			return(TRUE);
	}
	return(FALSE);
}

static char isOnlyOneWordPreCode(char *line, int wi) {
	int  i, wCnt;
	char word[BUFSIZ+1];

	findWholeScope(line, wi, templineC);
	uS.remFrontAndBackBlanks(templineC);
	if (templineC[0] == EOS)
		return(TRUE);
	wCnt = 0;
	i = 0;
	while ((i=getword(utterance->speaker, templineC, word, NULL, i))) {
		if (word[0] != '[')
			wCnt++;
	}
	if (wCnt <= 1)
		return(TRUE);
	else
		return(FALSE);
}

static char isMonoWordPreCode(char *line, int wi) {
	int  i, wCnt, syllCnt;
	char word[BUFSIZ+1];

	findWholeScope(line, wi, templineC);
	uS.remFrontAndBackBlanks(templineC);
	if (templineC[0] == EOS)
		return(TRUE);
	wCnt = 0;
	syllCnt = 0;
	i = 0;
	while ((i=getword("*", templineC, word, NULL, i))) {
		if (word[0] != '[') {
			wCnt++;
			syllCnt += countSyllables(word);
		}
	}
	if (wCnt <= 1) {
		if (syllCnt == 1)
			return(TRUE);
	}
	return(FALSE);
}

static char isPreviousItemWWR(char *line, int wi) {

	for (wi--; wi >= 0 && (isSpace(line[wi]) || line[wi] == '\n'); wi--) ;
	if (wi < 3)
		return(FALSE);
	if ((isSpace(line[wi-3]) || line[wi-3] == '\n') && line[wi-2] == '[' && line[wi-1] == '/' && line[wi] == ']')
		return(TRUE);
	return(FALSE);
}

static float roundFloat(double num) {
	long t;

	t = (long)num;
	num = num - t;
	if (num > 0.5)
		t++;
	return(t);
}

static int countMatchedWordSyms(char *word, char *nPat) {
	int i, len, cnt;

	cnt = 0;
	len = strlen(nPat);
	for (i=0; word[i] != EOS; i++) {
		if (strncmp(word+i, nPat, len) == 0)
			cnt++;
	}
	return(cnt);
}

static int countMatchedSyms(char *word, char *pat) {
	char isFound, nPat[4];

	isFound = FALSE;
	if (pat[0] == '*' || pat[0] == '%') {
		if (pat[1] == ':' || pat[1] == '^') {
			if (pat[2] == '*' || pat[2] == '%') {
				nPat[0] = pat[1];
				nPat[1] = EOS;
				return(countMatchedWordSyms(word, nPat));
			}
		} else if (UTF8_IS_LEAD((unsigned char)pat[1]) && pat[1] == (char)0xE2) {
			if (pat[2] == (char)0x89 && pat[3] == (char)0xA0) { // ≠
				if (pat[4] == '*' || pat[4] == '%') {
					nPat[0] = pat[1];
					nPat[1] = pat[2];
					nPat[2] = pat[3];
					nPat[3] = EOS;
					return(countMatchedWordSyms(word, nPat));
				}
			} else if (pat[2] == (char)0x86 && pat[3] == (char)0xAB) { // ↫ - - ↫
				if (pat[4] == '*' || pat[4] == '%') {
					nPat[0] = pat[1];
					nPat[1] = pat[2];
					nPat[2] = pat[3];
					nPat[3] = EOS;
					return(countMatchedWordSyms(word, nPat)/2);
				}
			}
		}
	}
	return(0);
}

static void addToPOSList(struct flucalc_speakers *ts, char *ws, char *wm) {

	rootLines = flucalc_AddNewLine(rootLines, NULL, ws);
	ts->total++;
	ts->words = flucalc_freqTree(ts->words, ts, wm, rootLines);
	if (rootLines != NULL && !rootLines->isUsed)
		rootLines = flucal_freeLines(rootLines, TRUE);

}

#define set_WWR_to_1(x)		(char)(x | 1)
#define is_WWR(x)			(char)(x & 1)

#define set_mWWR_to_1(x)	(char)(x | 2)
#define is_mWWR(x)			(char)(x & 2)

#define set_WWRRU_to_1(x)	(char)(x | 4)
#define is_WWRRU(x)			(char)(x & 4)

#define set_mWWRRU_to_1(x)	(char)(x | 8)
#define is_mWWRRU(x)		(char)(x & 8)

#define set_wRev_to_1(x)	(char)(x | 16)
#define is_wRev(x)			(char)(x & 16)

#define set_phRep_to_1(x)	(char)(x | 32)
#define is_phRep(x)			(char)(x & 32)

#define set_phRev_to_1(x)	(char)(x | 64)
#define is_phRev(x)			(char)(x & 64)

#define WWRSQMAX 200
// #define DEBUGmWWR

void call()	{		/* tabulate array of word lengths */
	int  i, j, wi, wsLen, syllCnt, chrCnt, wwrSqI, wwrSqCnt;
	unsigned char wwrSq[WWRSQMAX];
	char word[BUFSIZ+1], *ws, *wm, *s;
	char isRepSeg, isWWRFound, ismWWRFound;
	long stime, etime;
	double tNum;
	struct flucalc_speakers *ts;
	MORFEATS word_feats, *compd, *feat;
	IEWORDS *twd;

	if (isPOSMatch) {
		fprintf(stderr,"From file <%s>\n", oldfname);
	}
	ts = NULL;
	currentatt = 0;
	currentchar = (char)getc_cr(fpin, &currentatt);
	while (getwholeutter()) {
/*
 if (lineno > tlineno) {
	tlineno = lineno + 200;
 }
*/
		if (*utterance->speaker == '@') {
			if (uS.partcmp(utterance->speaker,"@ID:",FALSE,FALSE)) {
				if (isIDSpeakerSpecified(utterance->line, templineC, TRUE)) {
					uS.remblanks(utterance->line);
					flucalc_FindSpeaker(oldfname, templineC, utterance->line, FALSE);
				}
			}
		} else if (*utterance->speaker == '*') {
			strcpy(templineC, utterance->speaker);
			ts = flucalc_FindSpeaker(oldfname, templineC, NULL, TRUE);
			if (ts != NULL) {
				for (i=0; utterance->line[i] != EOS; i++) {

					if (utterance->line[i] == HIDEN_C && isdigit(utterance->line[i+1])) {
						if (getMediaTagInfo(utterance->line+i, &stime, &etime)) {
							tNum = etime;
							tNum = tNum - stime;
							tNum = tNum / 1000.0000;
							ts->tm = ts->tm + roundFloat(tNum);
						}
					}

					if ((i == 0 || uS.isskip(utterance->line,i-1,&dFnt,MBF)) && utterance->line[i] == '+' &&
						uS.isRightChar(utterance->line,i+1,',',&dFnt, MBF) && ts->isPSDFound) {
						if (ts->morUtt > 0.0)
							ts->morUtt--;
						ts->isPSDFound = FALSE;
					}
				}
				for (i=0; i < WWRSQMAX; i++) {
					wwrSq[i] = 0;
				}
				wwrSqCnt = 0;
				isWWRFound = FALSE;
				ismWWRFound = FALSE;
				i = 0;
				while ((i=getword(utterance->speaker, utterance->line, word, &wi, i))) {
					isRepSeg = FALSE;
					if (word[0] != '[' && word[0] != '0') {
						for (j=0; word[j] != EOS; j++) {
							if (word[j] == ':')
								ts->prolongation++;
							else if (word[j] == '^')
								ts->broken_word++;
							else if (UTF8_IS_LEAD((unsigned char)word[j]) && word[j] == (char)0xE2) {
								if (word[j+1] == (char)0x89 && word[j+2] == (char)0xA0) // ≠
									ts->block++;
								else if (word[j+1] == (char)0x86 && word[j+2] == (char)0xAB) { // ↫ - - ↫
									isRepSeg = !isRepSeg;
									if (isRepSeg) {
										ts->PWRRU++;
										ts->PWR++;
// fprintf(stdout, "PWR=%s\n", word); // lxs
									}
								}
							} else if (word[j] == '&' && word[j+1] == '+') {
								ts->phono_frag++;
							} else if (word[j] == '&' && word[j+1] == '-') {
								ts->filled_pauses++;
							}
							if (isRepSeg && word[j] == '-') // ↫ - - ↫
								ts->PWRRU++;
						}
					}
					if (!strcmp(word, "[/]")) {
						if (isOnlyOneWordPreCode(utterance->line, wi)) {
							if (isWWRFound == FALSE) {
								ts->WWR++;
								wwrSq[wwrSqCnt] = set_WWR_to_1(wwrSq[wwrSqCnt]);
								isWWRFound = TRUE;
							}
							ts->WWRRU++;
							wwrSq[wwrSqCnt] = set_WWRRU_to_1(wwrSq[wwrSqCnt]);
							if (isMonoWordPreCode(utterance->line, wi)) {
								if (ismWWRFound == FALSE) {
#ifdef DEBUGmWWR
fprintf(stdout, "1 (%d) mWWR=%s\n", wwrSqCnt, utterance->line+wi); // lxs
#endif
									ts->mWWR++;
									wwrSq[wwrSqCnt] = set_mWWR_to_1(wwrSq[wwrSqCnt]);
									ismWWRFound = TRUE;
								}
								ts->mWWRRU++;
								wwrSq[wwrSqCnt] = set_mWWRRU_to_1(wwrSq[wwrSqCnt]);
							}
						} else {
							wwrSq[wwrSqCnt] = set_phRep_to_1(wwrSq[wwrSqCnt]);
							ts->phrase_repets++;
						}
						wwrSqCnt++;
					} else {
						if (!isPreviousItemWWR(utterance->line, wi)) {
							isWWRFound = FALSE;
							ismWWRFound = FALSE;
						}
						if (!strcmp(word, "[//]")) {
							if (isOnlyOneWordPreCode(utterance->line, wi)) {
								ts->word_revis++;
								wwrSq[wwrSqCnt] = set_wRev_to_1(wwrSq[wwrSqCnt]);
							} else {
								ts->phrase_revis++;
								wwrSq[wwrSqCnt] = set_phRev_to_1(wwrSq[wwrSqCnt]);
							}
							wwrSqCnt++;
						} else if (word[0] == '(' && uS.isPause(word, 0, NULL, &j)) {
							ts->pauses_cnt++;
							ts->pauses_dur = ts->pauses_dur + getPauseTimeDuration(word);
						}
					}
				}
			}
			strcpy(spareTier1, utterance->line);
		} else if (*utterance->speaker == '%' && ts != NULL) {
			ts->isMORFound = TRUE;

			if (isPOSMatch) {
				wwrSqI = 0;
#ifdef DEBUGmWWR
strcpy(spareTier2, utterance->line); // lxs
removeDepTierItems(spareTier2); // lxs
				while ((i=getNextDepTierPair(utterance->line, word, templineC4, &wi, i)) != 0) {
for (; isSpace(spareTier2[wi]); wi++) ; // lxs
#else // DEBUGmWWR
				while ((i=getNextDepTierPair(utterance->line, word, templineC4, NULL, i)) != 0) {
#endif // DEBUGmWWR
						strcpy(templineC2, templineC4);
					if (!exclude(templineC2)) {
					} else if (word[0] != EOS && templineC4[0] != EOS) {
						if (isMorTierFirst) {
							wm = word;
							ws = templineC2;
						} else {
							wm = templineC2;
							ws = word;
						}
						for (twd=wdptr; twd != NULL; twd=twd->nextword) {
							strcpy(templineC2, templineC4);
							if (uS.patmat(templineC2, twd->word)) {
								wsLen = strlen(ws);
								if (!strcmp(templineC4, "[/]") || !strcmp(templineC4, "[//]")) {
									if (wwrSqI < wwrSqCnt) {
										if (is_WWR(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {WWR}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
										}
										if (is_mWWR(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {mono-WWR}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
#ifdef DEBUGmWWR
fprintf(stdout, "2 (%d) mWWR=%s\n", wwrSqI, spareTier2+wi); // lxs
#endif
										}
										if (is_WWRRU(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {WWR-RU}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
										}
										if (is_mWWRRU(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {mono-WWR-RU}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
										}
										if (is_phRep(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {Phrase repetition}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
										}
										if (is_wRev(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {Word revision}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
										}
										if (is_phRev(wwrSq[wwrSqI])) {
											ws[wsLen] = EOS;
											strcat(ws, " {Phrase revision}");
											addToPOSList(ts, ws, wm);
											ws[wsLen] = EOS;
										}
									}
								} else {
									chrCnt = countMatchedSyms(templineC4, twd->word);
									do {
										uS.remblanks(templineC2);
										ws[wsLen] = EOS;
										strcat(ws, " {");
										strcat(ws, twd->word);
										strcat(ws, "}");
										addToPOSList(ts, ws, wm);
										ws[wsLen] = EOS;
										chrCnt--;
									} while (chrCnt > 0) ;
								}
							}
						}
					}
					if (!strcmp(templineC4, "[/]") || !strcmp(templineC4, "[//]")) {
						wwrSqI++;
					}
				}
			} else {
				i = 0;
				while ((i=getNextDepTierPair(uttline, word, templineC4, NULL, i)) != 0) {
					if (word[0] != EOS && templineC4[0] != EOS) {
						if (strchr(word, '|') != NULL) {
							wm = word;
							ws = templineC4;
						} else {
							wm = templineC4;
							ws = word;
						}
						syllCnt = 0;
						s = strchr(wm, '|');
						if (s != NULL) {
							ts->morWords++;
							strcpy(templineC2, wm);
							if (ParseWordMorElems(templineC2, &word_feats) == FALSE)
								flucalc_error(FALSE);
							for (feat=&word_feats; feat != NULL; feat=feat->clitc) {
								// counts open/closed BEG
								if (isEqual("adv:int", feat->pos)) {
									ts->closedClass++;
								} else if (isEqual("n", feat->pos) || isnEqual("n:", feat->pos, 2) || isAllv(feat) || isnEqual("cop", feat->pos, 3) ||
										   isEqual("adj", feat->pos)) {
									ts->openClass++;
								} else if ((isEqual("adv", feat->pos) || isnEqual("adv:", feat->pos, 4)) &&
										   isEqualIxes("LY", feat->suffix, NUM_SUFF)) {
									ts->openClass++;
								} else if (isEqual("co", feat->pos) == FALSE && isEqual("on", feat->pos) == FALSE) {
									ts->closedClass++;
								}
								// counts open/closed END
								if (*(s+1) == '+') {
									if (feat->compd != NULL) {
										for (compd=feat; compd != NULL; compd=compd->compd) {
											if (compd->stem != NULL && compd->stem[0] != EOS) {
												syllCnt += countSyllables(compd->stem);
												syllCnt += countSyllables(ws+strlen(compd->stem));
												break;
											}
										}
										break;
									}
/*
									else {
										if (feat->stem != NULL && feat->stem[0] != EOS) {
											syllCnt += countSyllables(feat->stem);
										}
									}
*/
								}
							}
							freeUpFeats(&word_feats);
							if (syllCnt == 0) {
								syllCnt += countSyllables(ws);
							}
							if (SyllWordsListFP != NULL) {
								rootWords = flucalc_tree(rootWords, ws, syllCnt);
							}
							ts->morSyllables += (float)syllCnt;
						} else if (!isPOSMatch) {
							if (isTierContSymbol(wm, 0, FALSE))  //    +.  +/.  +/?  +//?  +...  +/.?   ===>   +,
								ts->isPSDFound = TRUE;
							for (j=0; wm[j] != EOS; j++) {
								if (isUttDel(wm, j)) {
									ts->morUtt = ts->morUtt + (float)1.0;
									break;
								}
							}
						}
					}
				}
			}
/*
			i = 0;
			while ((i=getword(utterance->speaker, uttline, word, NULL, i))) {
				if (strchr(word, '|') != NULL) {
					ts->morWords++;
// count Syllables beg
					strcpy(templineC2, word);
					if (ParseWordMorElems(templineC2, &word_feats) == FALSE)
						flucalc_error(FALSE);
					for (feat=&word_feats; feat != NULL; feat=feat->clitc) {
						if (feat->compd != NULL) {
							for (compd=feat; compd != NULL; compd=compd->compd) {
								if (compd->stem != NULL && compd->stem[0] != EOS) {
									ts->morSyllables += countSyllables(compd->stem);
								}
							}
						} else {
							if (feat->stem != NULL && feat->stem[0] != EOS) {
								ts->morSyllables += countSyllables(feat->stem);
							}
						}
					}
					freeUpFeats(&word_feats);
// count Syllables end
				} else {
					if (isTierContSymbol(word, 0, FALSE))  //    +.  +/.  +/?  +//?  +...  +/.?   ===>   +,
						ts->isPSDFound = TRUE;
					for (j=0; word[j] != EOS; j++) {
						if (isUttDel(word, j)) {
							ts->morUtt = ts->morUtt + (float)1.0;
							break;
						}
					}
				}
			}
*/
		}
	}
	if (!combinput) {
		flucalc_pr_result();
		rootLines = flucal_freeLines(rootLines, FALSE);
	}
}

static void flucalc_treeprint(struct flucalc_tnode *p) {
	if (p != NULL) {
		flucalc_treeprint(p->left);
		do {
			fprintf(SyllWordsListFP,"%s %d\n", p->word, p->count);
			if (p->right == NULL)
				break;
			if (p->right->left != NULL) {
				flucalc_treeprint(p->right);
				break;
			}
			p = p->right;
		} while (1);
	}
}

CLAN_MAIN_RETURN main(int argc, char *argv[]) {
	isWinMode = IS_WIN_MODE;
	CLAN_PROG_NUM = FLUCALC;
	chatmode = CHAT_MODE;
	OnlydataLimit = 1;
	UttlineEqUtterance = FALSE;
	rootWords = NULL;
	isSyllWordsList = FALSE;
	SyllWordsListFP = NULL;
	bmain(argc,argv,flucalc_pr_result);
	rootLines = flucal_freeLines(rootLines, FALSE);
	if (SyllWordsListFP != NULL) {
		flucalc_treeprint(rootWords);
		fclose(SyllWordsListFP);
	}
	flucalc_freetree(rootWords);
	rootWords = NULL;
	sp_head = freespeakers(sp_head);
}
